@@ -4,7 +4,6 @@ use serde::de::{
 };
 use serde::ser::{Serialize, Serializer};
 use serde::{forward_to_deserialize_any, Deserialize};
-use std::borrow::Cow;
 use std::fmt::{self, Debug, Display};
 use std::marker::PhantomData;
 use std::str::FromStr;
@@ -166,6 +165,7 @@ kind! {
     ModeAttr,
     NamespaceDecl,
     NoThrowAttr,
+    NoUniqueAddressAttr,
     NonNullAttr,
     NonTypeTemplateParmDecl,
     NullStmt,
@@ -308,18 +308,26 @@ impl std::error::Error for ParseKindError {}
 
 pub(crate) enum AnyKind<'de> {
     Kind(Kind),
-    Other(Cow<'de, str>),
+    Borrowed(&'de str),
+    Owned(Box<str>),
 }
 
 impl<'de> AnyKind<'de> {
     pub(crate) fn as_str(&self) -> &str {
         match self {
             AnyKind::Kind(kind) => kind.as_str(),
-            AnyKind::Other(kind) => kind.as_ref(),
+            AnyKind::Borrowed(kind) => kind,
+            AnyKind::Owned(kind) => kind,
         }
     }
 }
 
+impl<'de> Display for AnyKind<'de> {
+    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str(self.as_str())
+    }
+}
+
 impl<'de> Deserialize<'de> for AnyKind<'de> {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
     where
@@ -348,7 +356,7 @@ impl<'de> Visitor<'de> for AnyKindVisitor<'de> {
     {
         match Kind::deserialize(kind.into_deserializer()) {
             Ok(kind) => Ok(AnyKind::Kind(kind)),
-            Err(UnknownVariant) => Ok(AnyKind::Other(Cow::Owned(kind.to_owned()))),
+            Err(UnknownVariant) => Ok(AnyKind::Owned(Box::from(kind))),
         }
     }
 
@@ -358,7 +366,7 @@ impl<'de> Visitor<'de> for AnyKindVisitor<'de> {
     {
         match Kind::deserialize(kind.into_deserializer()) {
             Ok(kind) => Ok(AnyKind::Kind(kind)),
-            Err(UnknownVariant) => Ok(AnyKind::Other(Cow::Borrowed(kind))),
+            Err(UnknownVariant) => Ok(AnyKind::Borrowed(kind)),
         }
     }
 
@@ -368,7 +376,7 @@ impl<'de> Visitor<'de> for AnyKindVisitor<'de> {
     {
         match Kind::deserialize(kind.as_str().into_deserializer()) {
             Ok(kind) => Ok(AnyKind::Kind(kind)),
-            Err(UnknownVariant) => Ok(AnyKind::Other(Cow::Owned(kind))),
+            Err(UnknownVariant) => Ok(AnyKind::Owned(kind.into_boxed_str())),
         }
     }
 }
@@ -431,21 +439,33 @@ impl Display for UnknownVariant {
     }
 }
 
-pub(crate) struct BorrowedCowStrDeserializer<'a, 'de, E> {
-    value: &'a Cow<'de, str>,
+enum SometimesBorrowedStr<'a, 'de> {
+    Transient(&'a str),
+    Borrowed(&'de str),
+}
+
+pub(crate) struct SometimesBorrowedStrDeserializer<'a, 'de, E> {
+    value: SometimesBorrowedStr<'a, 'de>,
     error: PhantomData<E>,
 }
 
-impl<'a, 'de, E> BorrowedCowStrDeserializer<'a, 'de, E> {
-    pub(crate) fn new(value: &'a Cow<'de, str>) -> Self {
-        BorrowedCowStrDeserializer {
-            value,
+impl<'a, 'de, E> SometimesBorrowedStrDeserializer<'a, 'de, E> {
+    pub(crate) fn transient(value: &'a str) -> Self {
+        SometimesBorrowedStrDeserializer {
+            value: SometimesBorrowedStr::Transient(value),
+            error: PhantomData,
+        }
+    }
+
+    pub(crate) fn borrowed(value: &'de str) -> Self {
+        SometimesBorrowedStrDeserializer {
+            value: SometimesBorrowedStr::Borrowed(value),
             error: PhantomData,
         }
     }
 }
 
-impl<'a, 'de, E> Deserializer<'de> for BorrowedCowStrDeserializer<'a, 'de, E>
+impl<'a, 'de, E> Deserializer<'de> for SometimesBorrowedStrDeserializer<'a, 'de, E>
 where
     E: serde::de::Error,
 {
@@ -456,8 +476,8 @@ where
         V: Visitor<'de>,
     {
         match self.value {
-            Cow::Borrowed(string) => visitor.visit_borrowed_str(string),
-            Cow::Owned(string) => visitor.visit_str(string),
+            SometimesBorrowedStr::Transient(string) => visitor.visit_str(string),
+            SometimesBorrowedStr::Borrowed(string) => visitor.visit_borrowed_str(string),
         }
     }
 
@@ -489,7 +509,7 @@ where
     }
 }
 
-impl<'a, 'de, E> EnumAccess<'de> for BorrowedCowStrDeserializer<'a, 'de, E>
+impl<'a, 'de, E> EnumAccess<'de> for SometimesBorrowedStrDeserializer<'a, 'de, E>
 where
     E: serde::de::Error,
 {
