@@ -126,6 +126,7 @@ pub enum Clang {
     ModeAttr(ModeAttr),
     NamespaceDecl(NamespaceDecl),
     NoThrowAttr(NoThrowAttr),
+    NoUniqueAddressAttr(NoUniqueAddressAttr),
     NonNullAttr(NonNullAttr),
     NonTypeTemplateParmDecl(NonTypeTemplateParmDecl),
     NullStmt(NullStmt),
@@ -1591,6 +1592,13 @@ pub struct NoThrowAttr {
     pub implicit: bool,
 }
 
+#[derive(Deserialize, Debug)]
+#[serde(deny_unknown_fields)]
+#[non_exhaustive]
+pub struct NoUniqueAddressAttr {
+    pub range: SourceRange,
+}
+
 #[derive(Deserialize, Debug)]
 #[serde(deny_unknown_fields)]
 #[non_exhaustive]
@@ -2141,9 +2149,10 @@ pub struct UnresolvedMemberExpr {
 #[serde(deny_unknown_fields)]
 #[non_exhaustive]
 pub struct UnresolvedUsingValueDecl {
-    pub loc: SourceLocation,
-    pub range: SourceRange,
+    pub loc: Option<SourceLocation>,
+    pub range: Option<SourceRange>,
     pub name: Box<str>,
+    pub r#type: Option<Type>,
 }
 
 #[derive(Deserialize, Debug)]
